@@ -357,6 +357,7 @@ display_preferences_dialog (BonoboUIComponent *uic,
 
 	gtk_window_set_screen (GTK_WINDOW (fish->preferences_dialog),
 			       gtk_widget_get_screen (GTK_WIDGET (fish)));
+	gtk_window_set_resizable (GTK_WINDOW (fish->preferences_dialog), FALSE);
 	gtk_window_present (GTK_WINDOW (fish->preferences_dialog));
 
 	g_object_unref (xml);
@@ -433,7 +434,7 @@ display_about_dialog (BonoboUIComponent *uic,
 	gtk_window_set_screen (GTK_WINDOW (fish->about_dialog),
 			       gtk_widget_get_screen (GTK_WIDGET (fish)));
 	gnome_window_icon_set_from_file (GTK_WINDOW (fish->about_dialog),
-					 GNOME_ICONDIR " /gnome-fish.png");
+					 GNOME_ICONDIR "/gnome-fish.png");
 
 	g_signal_connect (fish->about_dialog, "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
@@ -1326,9 +1327,8 @@ handle_keypress (GtkWidget   *widget,
 }
 
 static gboolean 
-handle_button_press (GtkWidget      *widget,
-		     GdkEventButton *event,
-		     FishApplet     *fish)
+handle_button_press (FishApplet     *fish,
+		     GdkEventButton *event)
 {
 	if (event->button != 1)
 		return FALSE; 
@@ -1391,11 +1391,13 @@ setup_fish_widget (FishApplet *fish)
 	g_signal_connect (fish->drawing_area, "expose-event",
 			  G_CALLBACK (fish_applet_expose_event), fish);
 
-	gtk_widget_set_events (
-		fish->drawing_area,
-		gtk_widget_get_events (fish->drawing_area) | GDK_BUTTON_PRESS_MASK);
-	g_signal_connect (fish->drawing_area, "button_press_event",
-			  G_CALLBACK (handle_button_press), fish);
+	gtk_widget_add_events (widget, GDK_BUTTON_PRESS_MASK);
+	g_signal_connect_swapped (widget, "button_press_event",
+				  G_CALLBACK (handle_button_press), fish);
+
+	gtk_widget_add_events (fish->drawing_area, GDK_BUTTON_PRESS_MASK);
+	g_signal_connect_swapped (fish->drawing_area, "button_press_event",
+				  G_CALLBACK (handle_button_press), fish);
 
 	load_fish_image (fish);
 
