@@ -32,7 +32,7 @@
 
 #include "panel-util.h"
 #include "panel-config-global.h"
-#include "session.h"
+#include "panel-globals.h"
 
 #undef MENU_FENTRY_DEBUG
 
@@ -41,8 +41,6 @@
 
 static GSList *dir_list = NULL;
 
-extern GlobalConfig global_config;
-
 static GSList *
 prepend_mfile (GSList *list,
 	       const char *name,
@@ -224,6 +222,8 @@ fr_free (FileRec *fr, gboolean free_fr)
 	fr->comment = NULL;
 	g_free (fr->icon);
 	fr->icon = NULL;
+	g_free (fr->exec);
+	fr->exec = NULL;
 	g_free (fr->tryexec_path);
 	fr->tryexec_path = NULL;
 
@@ -277,7 +277,7 @@ fr_free (FileRec *fr, gboolean free_fr)
 static void
 fr_fill_dir (FileRec *fr, int sublevels)
 {
-	GSList *flist;
+	GSList *flist, *li;
 	DirRec *dr = (DirRec *)fr;
 	time_t curtime = time (NULL);
 	gboolean sorted = FALSE;
@@ -287,12 +287,10 @@ fr_fill_dir (FileRec *fr, int sublevels)
 	g_return_if_fail (fr->name != NULL);
 
 	flist = get_mfiles_from_menudir (fr->name, &sorted);
-	while (flist != NULL) {
-		MFile *mfile = flist->data;
+	for (li = flist; li != NULL; li = li->next) {
+		MFile *mfile = li->data;
 		char *name;
-		GSList *tmp = flist;
-		flist = flist->next;
-		g_slist_free_1 (tmp);
+		li->data = NULL;
 
 		if ( ! mfile->verified) {
 			free_mfile (mfile);
@@ -352,6 +350,8 @@ fr_fill_dir (FileRec *fr, int sublevels)
 				ffr->mtime = mfile->mtime;
 				ffr->last_stat = curtime;
 				ffr->parent = dr;
+				ffr->exec = qitem->exec;
+				qitem->exec = NULL;
 				ffr->icon = qitem->icon;
 				qitem->icon = NULL;
 				ffr->fullname = qitem->name;
@@ -373,6 +373,7 @@ fr_fill_dir (FileRec *fr, int sublevels)
 		g_free (name);
 		free_mfile (mfile);
 	}
+	g_slist_free (flist);
 	if (sorted)
 		dr->recs = g_slist_reverse (dr->recs);
 	else
@@ -438,42 +439,32 @@ fr_read_dir (DirRec *dr, const char *muri, time_t mtime, int sublevels)
 				     GNOME_VFS_FILE_INFO_DEFAULT)
 	    == GNOME_VFS_OK) {
 		QuickDesktopItem *qitem;
+
+		g_free (fr->icon);             fr->icon = NULL;
+		g_free (fr->fullname);         fr->fullname = NULL;
+		g_free (fr->name_collate_key); fr->name_collate_key = NULL;
+		g_free (fr->comment);          fr->comment = NULL;
+
 		qitem = quick_desktop_item_load_uri (furi /* uri */,
 						     NULL /* expected_type */,
 						     TRUE /* run_tryexec */);
-		if (qitem != NULL) {
-			g_free (fr->icon);
-			fr->icon = g_strdup (qitem->icon);
-
-			g_free (fr->fullname);
+		if (qitem) {
 			fr->fullname = g_strdup (qitem->name);
-
-			g_free (fr->name_collate_key);
-			fr->name_collate_key = NULL;
-
-			g_free (fr->comment);
-			fr->comment = g_strdup (qitem->comment);
+			fr->icon     = g_strdup (qitem->icon);
+			fr->comment  = g_strdup (qitem->comment);
 
 			quick_desktop_item_destroy (qitem);
-		} else {
-			g_free (fr->icon);
-			fr->icon = NULL;
-
-			g_free (fr->fullname);
-			fr->fullname = NULL;
-
-			g_free (fr->name_collate_key);
-			fr->name_collate_key = NULL;
-
-			g_free (fr->comment);
-			fr->comment = NULL;
 		}
+
 		/*if we statted*/
 		if (info->mtime != 0)
 			dr->ditemlast_stat = curtime;
 		dr->ditemmtime = info->mtime;
 	}
 	g_free (furi);
+
+	if (!fr->fullname)
+		fr->fullname = g_path_get_basename (muri);
 	
 	/* add if missing from list of directories */
 	if (g_slist_find (dir_list, fr) == NULL)
@@ -623,7 +614,7 @@ fr_check_and_reread (FileRec *fr)
 				}
 				if (ddr->ditemmtime != info->mtime) {
 					QuickDesktopItem *qitem;
-					qitem = quick_desktop_item_load_file (p /* file */,
+					qitem = quick_desktop_item_load_uri (p /* file */,
 									      NULL /* expected_type */,
 									      TRUE /* run_tryexec */);
 					if (qitem != NULL) {
@@ -671,10 +662,13 @@ fr_check_and_reread (FileRec *fr)
 				ffr->last_stat = curtime;
 				if (ffr->mtime != info->mtime) {
 					QuickDesktopItem *qitem;
-					qitem = quick_desktop_item_load_file (ffr->name /* file */,
-									      NULL /* expected_type */,
-									      TRUE /* run_tryexec */);
+					qitem = quick_desktop_item_load_uri (ffr->name /* file */,
+									     NULL /* expected_type */,
+									     TRUE /* run_tryexec */);
 					if (qitem != NULL) {
+						g_free (ffr->exec);
+						ffr->exec = g_strdup (qitem->exec);
+
 						g_free (ffr->icon);
 						ffr->icon = g_strdup (qitem->icon);
 
