@@ -153,12 +153,9 @@ panel_applet_associate_schemas_in_dir (GConfClient  *client,
 		gchar *subdir = l->data;
 		gchar *prefs_subdir;
 		gchar *schema_subdir;
-		gchar *tmp;
 
-		tmp = g_path_get_basename (subdir);
-
-		prefs_subdir  = g_strdup_printf ("%s/%s", prefs_key, tmp);
-		schema_subdir = g_strdup_printf ("%s/%s", schema_dir, tmp);
+		prefs_subdir  = g_strdup_printf ("%s/%s", prefs_key, subdir);
+		schema_subdir = g_strdup_printf ("%s/%s", schema_dir, subdir);
 
 		panel_applet_associate_schemas_in_dir (
 			client, prefs_subdir, schema_subdir, error);
@@ -166,7 +163,6 @@ panel_applet_associate_schemas_in_dir (GConfClient  *client,
 		g_free (prefs_subdir);
 		g_free (schema_subdir);
 		g_free (subdir);
-		g_free (tmp);
 
 		if (*error) {
 			g_slist_free (list);
@@ -187,7 +183,10 @@ panel_applet_add_preferences (PanelApplet  *applet,
 	GError       *our_error = NULL;
 
 	g_return_if_fail (PANEL_IS_APPLET (applet));
-	g_return_if_fail (applet->priv->prefs_key != NULL);
+	g_return_if_fail (schema_dir != NULL);
+
+	if (!applet->priv->prefs_key)
+		return;
 
 	if (opt_error)
 		error = opt_error;
